import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

//il tavoliere;
//contiene metodi per la lettura da file e per determinare la lista di posizioni
//giocabili dato il turno;
//una implementazione realistica dovrebbe includere anche metodi per effettuare
//le giocate e per determinare se uno dei giocatori ha vinto e/o se la partita e'
//terminata
public class Board {
	//costanti

	//caratteri usati nella rappresentazione del tavoliere
	public static final char WHITE_CHAR = 'B', BLACK_CHAR = 'N', EMPTY_CHAR = '.';
	//dimensione del tavoliere
	public static final int SIZE = 8;
	//valori usati per i turni e nel tavoliere
	//WHITE deve essere uguale a -BLACK e diverso da zero, EMPTY deve essere zero
	public static final int WHITE = 1, BLACK = -1, EMPTY = 0;

	//variabili di istanza	

	//array bidimensionale del tavoliere; conterra' elementi di valore WHITE, BLACK e EMPTY
	private int[][] positions = new int[SIZE][SIZE];

	//metodi

	//ritorna il contenuto del tavoliere alla riga e alla colonna specificate
	public int getPosition(int row, int col) {
		return positions[row][col];
	}

	//legge il tavoliere da un file di cui viene specificato il nome
	//se ci sono problemi in lettura o se il formato del file non e' valido
	//lancia una eccezione di tipo InvalidBoardException;
	//vengono lette solo le prime SIZE righe, se ce ne sono altre vengono ignorate
	public void readFromFile(String fileName) throws InvalidBoardException {
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(fileName));
			for(int row = 0; row < SIZE; row++) {
				String line = br.readLine();
				if(line.length() != SIZE) {
					throw new InvalidBoardException("Line too long or too short at row "+(row+1));
				}
				for(int col = 0; col < SIZE; col++) {
					char c = line.charAt(col);
					switch(c) {
						case WHITE_CHAR:
							positions[row][col] = WHITE;
							break;
						case BLACK_CHAR:
							positions[row][col] = BLACK;
							break;
						case EMPTY_CHAR:
							break;
						default:
							throw new InvalidBoardException("Invalid char "+c+" at row "+(row+1));
					}
				}
			}
		} catch(IOException e) {
			throw new InvalidBoardException("Error reading from file", e);
		} finally {
			if(br != null) {
				try {
					br.close();
				} catch (IOException e) {
					//ignorata
				}
			}
		}
	}

	//ritorna una lista contenente degli elementi di tipo Integer rappresentanti
	//le posizioni giocabili dal giocatore di turno specificato;
	//al fine dell'esercizio basterebbe stampare le posizioni appena trovate,
	//l'uso della lista permette una soluzione piu' generale
	public List playableList(int turn) {
		List positionsList = new ArrayList();
		for(int row = 0; row < SIZE; row++) {
			for(int col = 0; col < SIZE; col++) {
				if(isPlayable(row, col, turn)) {
					positionsList.add(new Integer(1+col+row*SIZE));
				}
			}
		}
		return positionsList;
	}

	//a partire dalla posizione specificata cerca nelle 8 direzioni (N, S, E, O, NE, NO, SE, SO)
	//se c'e' una sequenza di una o piu' pedine opposte seguite da una pedina del turno;
	//si ferma tornando true appena trova una sequenza valida
	protected boolean isPlayable(int row, int col, int turn) {
		//incrementi in X e in Y per le 8 direzioni
		int[] dirX = {0, 0, 1, -1, 1, -1, 1, -1};
		int[] dirY = {-1, 1, 0, 0, -1, -1, 1, 1};
		boolean found = false;
		for(int i = 0; i < 8 && !found; i++) {
			if(getPosition(row, col) == EMPTY && isSequence(row, col, dirX[i], dirY[i], turn)) {
				found = true;
			}
		}
		return found;
	}

	//a partire da una certa posizione cerca nella direzione specificata una pedina
	//del turno specificato preceduta da pedine del turno opposto;
	//se nel cercare la pedina trova uno spazio vuoto o esce dal tavoliere torna false, 
	//altrimenti torna true
	protected boolean isSequence(int row, int col, int dirX, int dirY, int turn) {
		row += dirY;
		col += dirX;
		int count = 0; //numero di pedine del turno opposto nella sequenza
		//ora ci muoviamo nella direzione specificata cercando la pedina del turno
		boolean done = false;
		boolean found = false;
		while(positionIsValid(row, col) && !done) {
			if(getPosition(row, col) == turn) {
				found = true;
				done = true;
			} else if(getPosition(row, col) == EMPTY) {
				done = true;
			} else {
				count++;
			}
			row += dirY;
			col += dirX;
		}
		if(count > 0 && found) {
			return true;
		} else {
			return false;
		}
	}

	//controlla se una determinata posizione e' all'interno del tavoliere
	protected boolean positionIsValid(int row, int col) {
		return row >= 0 && row < SIZE && col >= 0 && col < SIZE;
	}
}
